@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPUnit.
  *
@@ -10,8 +10,8 @@
 namespace PHPUnit;
 
 /**
- * Marker interface for PHPUnit exceptions.
+ * @internal This class is not covered by the backward compatibility promise for PHPUnit
  */
-interface Exception
+interface Exception extends \Throwable
 {
 }
