@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPUnit.
  *
@@ -7,20 +7,17 @@
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+namespace PHPUnit\Framework;
+
+use Countable;
 
 /**
  * A Test can be run and collect its results.
- *
- * @since      Interface available since Release 2.0.0
  */
-interface PHPUnit_Framework_Test extends Countable
+interface Test extends Countable
 {
     /**
      * Runs a test and collects its result in a TestResult instance.
-     *
-     * @param PHPUnit_Framework_TestResult $result
-     *
-     * @return PHPUnit_Framework_TestResult
      */
-    public function run(PHPUnit_Framework_TestResult $result = null);
+    public function run(TestResult $result = null): TestResult;
 }
