@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPUnit.
  *
@@ -13,78 +13,29 @@ use PHPUnit\Framework\Error\Error;
 use Throwable;
 
 /**
- * A TestFailure collects a failed test together with the caught exception.
+ * @internal This class is not covered by the backward compatibility promise for PHPUnit
  */
-class TestFailure
+final class TestFailure
 {
+    /**
+     * @var null|Test
+     */
+    private $failedTest;
+
+    /**
+     * @var Throwable
+     */
+    private $thrownException;
+
     /**
      * @var string
      */
     private $testName;
 
-    /**
-     * @var Test|null
-     */
-    protected $failedTest;
-
-    /**
-     * @var Throwable
-     */
-    protected $thrownException;
-
-    /**
-     * Constructs a TestFailure with the given test and exception.
-     *
-     * @param Test      $failedTest
-     * @param Throwable $t
-     */
-    public function __construct(Test $failedTest, $t)
-    {
-        if ($failedTest instanceof SelfDescribing) {
-            $this->testName = $failedTest->toString();
-        } else {
-            $this->testName = \get_class($failedTest);
-        }
-
-        if (!$failedTest instanceof TestCase || !$failedTest->isInIsolation()) {
-            $this->failedTest = $failedTest;
-        }
-
-        $this->thrownException = $t;
-    }
-
-    /**
-     * Returns a short description of the failure.
-     *
-     * @return string
-     */
-    public function toString()
-    {
-        return \sprintf(
-            '%s: %s',
-            $this->testName,
-            $this->thrownException->getMessage()
-        );
-    }
-
-    /**
-     * Returns a description for the thrown exception.
-     *
-     * @return string
-     */
-    public function getExceptionAsString()
-    {
-        return self::exceptionToString($this->thrownException);
-    }
-
     /**
      * Returns a description for an exception.
-     *
-     * @param Throwable $e
-     *
-     * @return string
      */
-    public static function exceptionToString(Throwable $e)
+    public static function exceptionToString(Throwable $e): string
     {
         if ($e instanceof SelfDescribing) {
             $buffer = $e->toString();
@@ -93,6 +44,10 @@ class TestFailure
                 $buffer .= $e->getComparisonFailure()->getDiff();
             }
 
+            if ($e instanceof PHPTAssertionFailedError) {
+                $buffer .= $e->getDiff();
+            }
+
             if (!empty($buffer)) {
                 $buffer = \trim($buffer) . "\n";
             }
@@ -112,11 +67,49 @@ class TestFailure
     }
 
     /**
-     * Returns the name of the failing test (including data set, if any).
+     * Constructs a TestFailure with the given test and exception.
      *
-     * @return string
+     * @param Throwable $t
      */
-    public function getTestName()
+    public function __construct(Test $failedTest, $t)
+    {
+        if ($failedTest instanceof SelfDescribing) {
+            $this->testName = $failedTest->toString();
+        } else {
+            $this->testName = \get_class($failedTest);
+        }
+
+        if (!$failedTest instanceof TestCase || !$failedTest->isInIsolation()) {
+            $this->failedTest = $failedTest;
+        }
+
+        $this->thrownException = $t;
+    }
+
+    /**
+     * Returns a short description of the failure.
+     */
+    public function toString(): string
+    {
+        return \sprintf(
+            '%s: %s',
+            $this->testName,
+            $this->thrownException->getMessage()
+        );
+    }
+
+    /**
+     * Returns a description for the thrown exception.
+     */
+    public function getExceptionAsString(): string
+    {
+        return self::exceptionToString($this->thrownException);
+    }
+
+    /**
+     * Returns the name of the failing test (including data set, if any).
+     */
+    public function getTestName(): string
     {
         return $this->testName;
     }
@@ -128,30 +121,24 @@ class TestFailure
      * isolation.
      *
      * @see Exception
-     *
-     * @return Test|null
      */
-    public function failedTest()
+    public function failedTest(): ?Test
     {
         return $this->failedTest;
     }
 
     /**
      * Gets the thrown exception.
-     *
-     * @return Throwable
      */
-    public function thrownException()
+    public function thrownException(): Throwable
     {
         return $this->thrownException;
     }
 
     /**
      * Returns the exception's message.
-     *
-     * @return string
      */
-    public function exceptionMessage()
+    public function exceptionMessage(): string
     {
         return $this->thrownException()->getMessage();
     }
@@ -159,11 +146,9 @@ class TestFailure
     /**
      * Returns true if the thrown exception
      * is of type AssertionFailedError.
-     *
-     * @return bool
      */
-    public function isFailure()
+    public function isFailure(): bool
     {
-        return ($this->thrownException() instanceof AssertionFailedError);
+        return $this->thrownException() instanceof AssertionFailedError;
     }
 }
