@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPUnit.
  *
@@ -10,15 +10,10 @@
 namespace PHPUnit\Framework;
 
 /**
- * A warning.
+ * @internal This class is not covered by the backward compatibility promise for PHPUnit
  */
-class WarningTestCase extends TestCase
+final class WarningTestCase extends TestCase
 {
-    /**
-     * @var string
-     */
-    protected $message = '';
-
     /**
      * @var bool
      */
@@ -39,6 +34,11 @@ class WarningTestCase extends TestCase
      */
     protected $useErrorHandler = false;
 
+    /**
+     * @var string
+     */
+    private $message;
+
     /**
      * @param string $message
      */
@@ -48,29 +48,24 @@ class WarningTestCase extends TestCase
         parent::__construct('Warning');
     }
 
-    /**
-     * @throws Exception
-     */
-    protected function runTest()
-    {
-        throw new Warning($this->message);
-    }
-
-    /**
-     * @return string
-     */
-    public function getMessage()
+    public function getMessage(): string
     {
         return $this->message;
     }
 
     /**
      * Returns a string representation of the test case.
-     *
-     * @return string
      */
-    public function toString()
+    public function toString(): string
     {
         return 'Warning';
     }
+
+    /**
+     * @throws Exception
+     */
+    protected function runTest(): void
+    {
+        throw new Warning($this->message);
+    }
 }
