@@ -1,7 +1,15 @@
-<?php
-use PHPUnit\Framework\TestResult;
-use PHPUnit\Framework\TestCase;
+<?php declare(strict_types=1);
+/*
+ * This file is part of PHPUnit.
+ *
+ * (c) Sebastian Bergmann <sebastian@phpunit.de>
+ *
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
+ */
 use PHPUnit\Framework\Test;
+use PHPUnit\Framework\TestCase;
+use PHPUnit\Framework\TestResult;
 
 class DoubleTestCase implements Test
 {
@@ -17,7 +25,7 @@ class DoubleTestCase implements Test
         return 2;
     }
 
-    public function run(TestResult $result = null)
+    public function run(TestResult $result = null): TestResult
     {
         $result->startTest($this);
 
@@ -25,5 +33,7 @@ class DoubleTestCase implements Test
         $this->testCase->runBare();
 
         $result->endTest($this, 0);
+
+        return $result;
     }
 }
