@@ -1,10 +1,18 @@
-<?php
+<?php declare(strict_types=1);
+/*
+ * This file is part of PHPUnit.
+ *
+ * (c) Sebastian Bergmann <sebastian@phpunit.de>
+ *
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
+ */
 use PHPUnit\Framework\ExpectationFailedException;
 use PHPUnit\Framework\TestCase;
 
 class ExceptionStackTest extends TestCase
 {
-    public function testPrintingChildException()
+    public function testPrintingChildException(): void
     {
         try {
             $this->assertEquals([1], [2], 'message');
@@ -15,7 +23,7 @@ class ExceptionStackTest extends TestCase
         }
     }
 
-    public function testNestedExceptions()
+    public function testNestedExceptions(): void
     {
         $exceptionThree = new Exception('Three');
         $exceptionTwo   = new InvalidArgumentException('Two', 0, $exceptionThree);
