@@ -1,9 +1,11 @@
-<?php
-/**
- * Sample class that implements ArrayAccess copied from
- * http://www.php.net/manual/en/class.arrayaccess.php
- * with some minor changes
- * This class required for PHPUnit_Framework_Constraint_ArrayHasKey testing
+<?php declare(strict_types=1);
+/*
+ * This file is part of PHPUnit.
+ *
+ * (c) Sebastian Bergmann <sebastian@phpunit.de>
+ *
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
  */
 class SampleArrayAccess implements ArrayAccess
 {
@@ -13,24 +15,28 @@ class SampleArrayAccess implements ArrayAccess
     {
         $this->container = [];
     }
-    public function offsetSet($offset, $value)
+
+    public function offsetSet($offset, $value): void
     {
-        if (is_null($offset)) {
+        if (null === $offset) {
             $this->container[] = $value;
         } else {
             $this->container[$offset] = $value;
         }
     }
+
     public function offsetExists($offset)
     {
         return isset($this->container[$offset]);
     }
-    public function offsetUnset($offset)
+
+    public function offsetUnset($offset): void
     {
         unset($this->container[$offset]);
     }
+
     public function offsetGet($offset)
     {
-        return isset($this->container[$offset]) ? $this->container[$offset] : null;
+        return $this->container[$offset] ?? null;
     }
 }
