@@ -1,16 +1,16 @@
-<?php
+<?php declare(strict_types=1);
+/*
+ * This file is part of PHPUnit.
+ *
+ * (c) Sebastian Bergmann <sebastian@phpunit.de>
+ *
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
+ */
 class Singleton
 {
     private static $uniqueInstance = null;
 
-    protected function __construct()
-    {
-    }
-
-    final private function __clone()
-    {
-    }
-
     public static function getInstance()
     {
         if (self::$uniqueInstance === null) {
@@ -19,4 +19,12 @@ class Singleton
 
         return self::$uniqueInstance;
     }
+
+    protected function __construct()
+    {
+    }
+
+    private function __clone()
+    {
+    }
 }
