@@ -1,8 +1,13 @@
-<?php
+<?php declare(strict_types=1);
+/*
+ * This file is part of PHPUnit.
+ *
+ * (c) Sebastian Bergmann <sebastian@phpunit.de>
+ *
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
+ */
 
-/* This class is used for testing a chain of IteratorAggregate objects, since
- * PHP does allow IteratorAggregate::getIterator() to return an instance of the
- * same class. */
 class TestIteratorAggregate2 implements IteratorAggregate
 {
     private $traversable;
