@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPUnit.
  *
@@ -14,7 +14,10 @@ use PHPUnit\Framework\Constraint\SameSize;
 use PHPUnit\Framework\Constraint\TraversableContains;
 use PHPUnit\Util\Filter;
 
-class ConstraintTest extends TestCase
+/**
+ * @small
+ */
+final class ConstraintTest extends TestCase
 {
     public function testConstraintArrayNotHasKey(): void
     {
@@ -995,6 +998,9 @@ EOF
         $this->fail();
     }
 
+    /**
+     * @testdox Constraint PCRE not match
+     */
     public function testConstraintPCRENotMatch(): void
     {
         $constraint = Assert::logicalNot(
@@ -1024,6 +1030,9 @@ EOF
         $this->fail();
     }
 
+    /**
+     * @testdox Constraint PCRE not match with custom message
+     */
     public function testConstraintPCRENotMatch2(): void
     {
         $constraint = Assert::logicalNot(
@@ -1288,66 +1297,6 @@ EOF
 custom message
 Failed asserting that an array does not contain 'foo'.
 
-EOF
-                ,
-                TestFailure::exceptionToString($e)
-            );
-
-            return;
-        }
-
-        $this->fail();
-    }
-
-    public function testAttributeNotEqualTo(): void
-    {
-        $object     = new \ClassWithNonPublicAttributes;
-        $constraint = Assert::logicalNot(
-            Assert::attributeEqualTo('foo', 2)
-        );
-
-        $this->assertTrue($constraint->evaluate($object, '', true));
-        $this->assertEquals('attribute "foo" is not equal to 2', $constraint->toString());
-        $this->assertCount(1, $constraint);
-
-        $constraint = Assert::logicalNot(
-            Assert::attributeEqualTo('foo', 1)
-        );
-
-        $this->assertFalse($constraint->evaluate($object, '', true));
-
-        try {
-            $constraint->evaluate($object);
-        } catch (ExpectationFailedException $e) {
-            $this->assertEquals(
-                <<<EOF
-Failed asserting that attribute "foo" is not equal to 1.
-
-EOF
-                ,
-                TestFailure::exceptionToString($e)
-            );
-
-            return;
-        }
-
-        $this->fail();
-    }
-
-    public function testAttributeNotEqualTo2(): void
-    {
-        $object     = new \ClassWithNonPublicAttributes;
-        $constraint = Assert::logicalNot(
-            Assert::attributeEqualTo('foo', 1)
-        );
-
-        try {
-            $constraint->evaluate($object, 'custom message');
-        } catch (ExpectationFailedException $e) {
-            $this->assertEquals(
-                <<<EOF
-custom message\nFailed asserting that attribute "foo" is not equal to 1.
-
 EOF
                 ,
                 TestFailure::exceptionToString($e)
